<?php

/**
 * @file
 * Admin options for SMTP servers
 */


/**
 * Global settings form for SMTP usage.
 * 
 * This form replicates the basic SMTP options from the SMTP module that are
 * *not* specific to a single server.
 */
function multi_smtp_admin_settings_form(&$form_state) {
  $form = array();
  $form['smtp_on'] = array(
    '#type' => 'radios',
    '#title' => t('Turn this module on or off'),
    '#default_value' => variable_get('smtp_on', 0),
    '#options' => array(1 => t('On'), 0 => t('Off')),
    '#description' => t('To uninstall this module you must turn it off here first.'),
  );

  $form['smtp_debugging'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable debugging'),
    '#default_value' => variable_get('smtp_debugging', 0),
    '#description' => t("Checking this box will print SMTP messages from the server for every e-mail that is sent. This should never be enabled for production usage; it results in so much information that it should only be used when something isn't working."),
  );

  $form['multi_smtp_server_wait_time'] = array(
    '#type' => 'select',
    '#title' => t('Server wait time'),
    '#description' => t('Amount of time to put a server on hold if it fails to send.'),
    '#default_value' => variable_get('multi_smtp_server_wait_time', MULTI_SMTP_DEFAULT_SERVER_WAIT_TIME),
    '#options' => drupal_map_assoc(array(15, 30, 60, 120, 180, 240, 300, 600, 900, 1200, 1800, 3600, 7200, 14400, 28800, 43200, 86400, 172800, 259200, 345600, 604800), 'format_interval'),
  );

  return system_settings_form($form);
}


/**
 * Form to send a test message on a server.
 *
 * Takes an e-mail address and a selection of a server, and sends a test e-mail
 * from that server to that address.
 */
function multi_smtp_admin_server_test_form(&$form_state) {
  $form = array();

  $form['test_email'] = array(
    '#type' => 'textfield',
    '#title' => t('E-mail Address'),
    '#description' => t('E-mail address to send the test message.'),
    '#size' => 40,
    '#maxlength' => 255,
    '#required' => TRUE,
  );

  // Get all servers, including those that are disabled.
  $servers = multi_smtp_get_all_servers(TRUE);

  // If no servers have been setup, you can't send a test message.
  if (empty($servers)) {
    drupal_set_message(t('No SMTP servers have been configured, so no test message can be sent. Click on the Add Server tab to setup a server.'), 'error');
    return '';
  }

  $options = array(
    '' => '- ' . t('Choose one') . ' -',
    t('Active') => array(),
    t('Disabled') => array()
  );
  foreach ($servers as $name => $server) {
    $group = empty($server->disabled) ? t('Active') : t('Disabled');
    $options[$group][$name] = t('@smtp_nickname (@smtp_host)', array(
      '@smtp_nickname' => $server->admin_title,
      '@smtp_host' => $server->smtp_host,
    ));
  }

  $form['server'] = array(
    '#type' => 'select',
    '#title' => t('Use server'),
    '#description' => t('Select the SMTP server to use.'),
    '#options' => $options,
    '#default_value' => -1,
    '#required' => TRUE
  );

  $form['send'] = array(
    '#type' => 'submit',
    '#value' => t('Send'),
  );

  return $form;
}


/**
 * Validate the test send form.
 */
function multi_smtp_admin_server_test_form_validate($form, &$form_state) {
  if ($form_state['values']['test_email'] && !valid_email_address($form_state['values']['test_email'])) {
    form_set_error('test_email', t("Enter a valid e-mail address."));
  }
}


/**
 * Send a test message.
 */
function multi_smtp_admin_server_test_form_submit($form, &$form_state) {
  // If an address was given, send a test e-mail message.
  $test_address = $form_state['values']['test_email'];
  multi_smtp_load_vars($form_state['values']['server']);
  global $language;
  $params['subject'] = t('Drupal test e-mail');
  $params['body']    = t('If you receive this message it means your site is capable of sending e-mail. Sent from server @smtp_nickname (@smtp_host)', array('@smtp_nickname' => variable_get('smtp_nickname', ''), '@smtp_host' => variable_get('smtp_host', '')));
  drupal_mail('smtp', 'smtp-test', $test_address, $language, $params);
  drupal_set_message(t('A test e-mail has been sent to @email. You may want to !check for any error messages.', array('@email' => $test_address, '!check' => l(t('check the logs'), 'admin/reports/dblog'))));
}
